/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointRegistryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.RouterMonitorServiceClient;
import org.ow2.petals.jmx.api.api.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.DefectListenerService;
import org.ow2.petals.jmx.api.api.monitoring.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.exception.NoDefectListenerException;
import org.ow2.petals.jmx.api.impl.monitoring.LocalTransporterMonitoringServiceClientImpl;
import org.ow2.petals.jmx.api.impl.monitoring.component.framework.ComponentMonitoringServiceClientImpl;
import org.ow2.petals.jmx.api.impl.monitoring.component.soap.PetalsBcSoapMonitoringDefectListenerImpl;
import org.ow2.petals.jmx.api.impl.monitoring.component.soap.PetalsBcSoapMonitoringServiceClientImpl;

/**
 * The Petals JMX Client API.
 * @author Christophe Deneux - Capgemini Sud
 * @author Cedric Girodengo - Capgemini Sud
 */
public final class JMXClientImpl implements JMXClient {

    private static final String PETALS_DOMAIN = "Petals";

    private static final String OW2_PETALS_DOMAIN = "org.ow2.petals";

    private static final String JAVA_LANG_DOMAIN = "java.lang";

    private static final String SUBSCRIPTION_ID_FORMAT = "%s_%s";

    private static final String BCSOAP_SUBSCRIPTION_PREFIX = "BCSOAP";

    private JMXConnection jmxConnection = null;

    private InetAddress jmxHost = null;

    private int jmxPort = 0;

    private String jmxUsername = null;

    private final Map<String, DefectListenerService> defectSubscriptions = new ConcurrentHashMap<String, DefectListenerService>();

    /**
     * Creates a new instance of {@link JMXClient} to a remote JMX server.
     * 
     * @param host
     *            Hostname of the remote JMX server. If set to <code>null</code>, the host name is
     *            <code>localhost</code>.
     * @param jndiPort
     *            JMX port of the remote JMX server
     * @param username
     *            Username required to establish a JMX connection to the remote JMX server
     * @param password
     *            Password required to establish a JMX connection to the remote JMX server
     * @throws ConnectionErrorException
     */
    public JMXClientImpl(String host, Integer jndiPort, String username, String password)
            throws ConnectionErrorException {
        try {
            this.jmxHost = InetAddress.getByName(host);
            if (jndiPort == null) {
                this.jmxPort = JMXClient.DEFAULT_PORT;
            } else {
                this.jmxPort = jndiPort.intValue();
            }
            this.jmxUsername = username;
            this.jmxConnection = new JMXConnection(this.jmxHost.getHostName(), this.jmxPort, username, password);
        } catch (final UnknownHostException e) {
            throw new ConnectionErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link JMXClient} to an embedded JMX server.
     * 
     * @param mbeanServer
     *            The embedded JMX server
     * @throws ConnectionErrorException
     */
    public JMXClientImpl(final MBeanServer mbeanServer) throws ConnectionErrorException {
        this.jmxConnection = new JMXConnection(mbeanServer);
    }

    @Override
    public InetAddress getHost() throws NotRemoteJMXClientException {
        if (this.jmxHost != null) {
            return this.jmxHost;
        } else {
            throw new NotRemoteJMXClientException();
        }
    }

    @Override
    public int getPort() throws NotRemoteJMXClientException {
        if (this.jmxHost != null) {
            return this.jmxPort;
        } else {
            throw new NotRemoteJMXClientException();
        }
    }

    @Override
    public String getUsername() throws NotRemoteJMXClientException {
        if (this.jmxUsername != null) {
            return this.jmxUsername;
        } else {
            throw new NotRemoteJMXClientException();
        }
    }

    /**
     * Retrieve the Petals Admin service client API
     * 
     * @return the Petals Admin service client API
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
            ConnectionErrorException {

        return new PetalsAdminServiceClientImpl(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the Admin service client API
     * 
     * @return the Admin service client API
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException,
            AdminServiceErrorException, ConnectionErrorException {

        return new AdminServiceClientImpl(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the installation service client API
     * 
     * @return the installation service client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public InstallationServiceClient getInstallationServiceClient()
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException,
            ConnectionErrorException {

        return new InstallationServiceClientImpl(PETALS_DOMAIN,
                jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the deployment service client API
     * 
     * @return the deployment service client API
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     * @throws ConnectionErrorException
     */
    public DeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException,
            ConnectionErrorException {

        return new DeploymentServiceClientImpl(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the component client API associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client API
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new ComponentClientImpl(OW2_PETALS_DOMAIN, componentName,
                jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the component client API associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client API
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(final ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        if (!componentMBeanName.getDomain().equals(OW2_PETALS_DOMAIN)) {
            throw new ComponentDoesNotExistException("Domain name is not '" + OW2_PETALS_DOMAIN + "'");
        }

        return new ComponentClientImpl(componentMBeanName, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the component installer client API associated to a JBI
     * component.
     * 
     * @param componentName
     *            Component name
     * @return the installer component client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public InstallerComponentClient getInstallerComponentClient(String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new InstallerComponentClientImpl(PETALS_DOMAIN, componentName,
                jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the EndpointRegistry client API
     * 
     * @return the EndpointRegistry component client API
     * @throws ConnectionErrorException
     * @throws EndpointRegistryServiceErrorException
     * @throws EndpointRegistryServiceDoesNotExistException
     */
    public EndpointRegistryClient getEndpointRegistryClient() throws ConnectionErrorException,
            EndpointRegistryServiceDoesNotExistException, EndpointRegistryServiceErrorException {

        return new EndpointRegistryClientImpl(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());
    }

    /**
     * Retrieve the system monitoring service client API
     * 
     * @return the SystemMonitoring component client API
     * @throws ConnectionErrorException
     * @throws SystemMonitoringServiceErrorException
     * @throws SystemMonitoringDoesNotExistException
     */
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient()
            throws ConnectionErrorException, SystemMonitoringDoesNotExistException,
            SystemMonitoringServiceErrorException {

        return new SystemMonitoringServiceClientImpl(JAVA_LANG_DOMAIN,
                jmxConnection
                .getMBeanServerConnection());

    }

    public RuntimeConfigurationComponentClient getRuntimeConfigurationComponentClient(
            final String dynamicComponentName) throws RuntimeConfigurationDoesNotExistException,
            RuntimeConfigurationErrorException, ConnectionErrorException {
        return new RuntimeConfigurationComponentClientImpl(OW2_PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection(), dynamicComponentName);
    }
    
    /**
     * Retrieve the Router Monitor service client API
     * 
     * @return The Router Monitor service client API
     * @throws Router MonitorServiceDoesNotExistException
     * @throws Router MonitorServiceErrorException
     * @throws ConnectionErrorException
     */
    public RouterMonitorServiceClient getRouterMonitoringServiceClient()
            throws RouterMonitorServiceDoesNotExistException, RouterMonitorServiceErrorException,
            ConnectionErrorException {

        return new RouterMonitorServiceClientImpl(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());
    }
    
    /**
     * Retrieve the Logger service client API
     * @return The Logger service client API
     * @throws LoggerServiceErrorException
     * @throws LoggerDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LoggerServiceClient getLoggerServiceClient() throws LoggerServiceErrorException,
            LoggerServiceDoesNotExistException, ConnectionErrorException {
        return new LoggerServiceClientImpl(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());
    }

    /**
     * Retrieve the local transporter monitoring service client API
     * 
     * @return The Logger service client API
     * @throws LocalTransporterMonitoringServiceErrorException
     * @throws LocalTransporterMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return new LocalTransporterMonitoringServiceClientImpl(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());
    }

    @Override
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(
            String componentIdentifier) throws ComponentMonitoringServiceErrorException,
            ComponentMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return new ComponentMonitoringServiceClientImpl(OW2_PETALS_DOMAIN, componentIdentifier,
                jmxConnection.getMBeanServerConnection());
    }

    /**
     * Retrieve the Petals BC SOAP monitoring service client API
     * 
     * @param componentIdentifier
     *            The unique identifier of the Petals BC SOAP
     * 
     * @return The Logger service client API
     * @throws ComponentMonitoringServiceErrorException
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(
            final String componentIdentifier) throws ComponentMonitoringServiceErrorException,
            ComponentMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return new PetalsBcSoapMonitoringServiceClientImpl(OW2_PETALS_DOMAIN, componentIdentifier,
                jmxConnection.getMBeanServerConnection());
    }

    public void subscribeToComponentMonitoringService(
            final String componentIdentifier, final DefectListener defectListener,
            final DefectFilter defectFilter) throws DefectListenerException,
            DefectEmitterDoesNotExistsException {
        
        try {
            final DefectListenerService defectListenerService = new PetalsBcSoapMonitoringDefectListenerImpl(
                    OW2_PETALS_DOMAIN, componentIdentifier, this.getHost(),
                    jmxConnection.getMBeanServerConnection());
            this.defectSubscriptions.put(String.format(SUBSCRIPTION_ID_FORMAT,
                    BCSOAP_SUBSCRIPTION_PREFIX, componentIdentifier), defectListenerService);
            defectListenerService.subscribe(defectListener, defectFilter);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new DefectEmitterDoesNotExistsException(e, componentIdentifier);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new DefectListenerException(e);
        } catch (final NotRemoteJMXClientException e) {
            throw new DefectListenerException(e);
        }
    }

    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException,
            DefectListenerException {
        
        final DefectListenerService defectListenerService = this.defectSubscriptions.remove(String
                .format(SUBSCRIPTION_ID_FORMAT, BCSOAP_SUBSCRIPTION_PREFIX, componentIdentifier));
        if (defectListenerService != null) {
            defectListenerService.unsubscribe();
        }
    }

    /**
     * {@inheritDoc}
     */
    public TopologyServiceClient getTopologyServiceClient() throws TopologyServiceErrorException,
            TopologyServiceDoesNotExistException, ConnectionErrorException {
        return new TopologyServiceClientImpl(PETALS_DOMAIN,
                this.jmxConnection.getMBeanServerConnection());
    }

    /**
     * Release the connection to the JMX server
     * 
     * @throws ConnectionErrorException
     */
    public void disconnect() throws ConnectionErrorException {
        this.jmxConnection.disconnect();
    }

    /**
     * Get the instance to the @ MBeanServerConnection} in use.
     * 
     * @throws ConnectionErrorException
     */
    public MBeanServerConnection getMBeanServerConnection() throws ConnectionErrorException {
        return this.jmxConnection.getMBeanServerConnection();
    }
}
