/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.jmx.api.api.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationErrorException;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class RuntimeConfigurationComponentClientImpl extends AbstractServiceClient implements
        RuntimeConfigurationComponentClient {

    private static final String RUNTIME_CONFIGURATION_JMX_NAME = "runtime_configuration_";

    private static final String RUNTIME_CONFIGURATION_JMX_TYPE = "custom";

    public RuntimeConfigurationComponentClientImpl(final String petalsDomain,
            final MBeanServerConnection beanServerConnection, final String componentName)
            throws RuntimeConfigurationDoesNotExistException, RuntimeConfigurationErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();

            /*
             * The RUNTIME_CONFIGURATION_JMX_NAME represent the static name part
             * of the focused MBean like "runtime_configuration_", and
             * dynamicComponentName the dynamic name part of this MBean like
             * "petals-bc-filetransfer". finally all the MBean can be:
             * runtime_configuration_petals-bc-filetransfer
             */
            attributes.put(JMX_FIELD_NAME, RUNTIME_CONFIGURATION_JMX_NAME + componentName);
            attributes.put(JMX_FIELD_TYPE, RUNTIME_CONFIGURATION_JMX_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new RuntimeConfigurationDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes()
            throws RuntimeConfigurationErrorException {
        try {
            final Map<MBeanAttributeInfo, Object> attributes = new HashMap<MBeanAttributeInfo, Object>();
            final MBeanAttributeInfo[] attrs = this.mBeanServerConnection.getMBeanInfo(this.mbeanName).getAttributes();
            for (final MBeanAttributeInfo info : attrs) {
                attributes.put(info, this.mBeanServerConnection.getAttribute(this.mbeanName, info
                        .getName()));
            }
            return attributes;

        } catch (final InstanceNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final IntrospectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final ReflectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final AttributeNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final MBeanException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
    }

    @Override
    public void setAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws RuntimeConfigurationErrorException {
        final AttributeList attributesList = new AttributeList();
        for (final Entry<MBeanAttributeInfo, Object> entry : attributes.entrySet()) {
            attributesList.add(new Attribute(entry.getKey().getName(), entry.getValue()));
        }
        try {
            this.mBeanServerConnection.setAttributes(this.mbeanName, attributesList);

        } catch (final InstanceNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final ReflectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (final IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
    }
}
