/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanAttributeInfo;
import javax.management.ObjectName;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.ConfigurationInstallerComponentClient;
import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.impl.mbean.ConfigurationInstallerComponentService;
import org.ow2.petals.jmx.api.impl.mbean.Installer;
import org.ow2.petals.jmx.api.impl.mbean.InstallerResults;

/**
 * <p>
 * Unit tests of {@link ConfigurationInstallerComponentClientImpl} to check that JMX client interfaces are aligned with
 * the JMX server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link ConfigurationInstallerComponentClient} to avoid to forget some methods. A unit test
 * will only invoke the associated method of {@link ConfigurationInstallerComponentClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationInstallerComponentClientImplTest extends AbstractServiceClientImpl implements
        ConfigurationInstallerComponentClient {

    private ConfigurationInstallerComponentClient configurationInstallerComponentServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerInstallerComponentService(
                new Installer(new InstallerResults().setGetInstallerConfigurationMBeanResult(new ObjectName(
                        "org.ow2.petals:type=custom,name=bootstrap_" + COMPONENT_ID))), COMPONENT_ID);
        this.embeddedJmxSrvCon.registerConfigurationInstallerComponentService(
                new ConfigurationInstallerComponentService(), COMPONENT_ID);
        this.configurationInstallerComponentServiceClient = jmxClient.getInstallerComponentClient(COMPONENT_ID)
                .getConfigurationInstallerClient();
    }

    /**
     * Check the interface consistency of {@link ConfigurationInstallerComponentClientImpl#getAttribute(String)}
     */
    @Test
    public void testGetAttribute() throws AttributeErrorException {

        this.getAttribute(ConfigurationInstallerComponentService.ATTR_NAME);
    }

    @Override
    public Object getAttribute(final String attribute) throws AttributeErrorException {

        return this.configurationInstallerComponentServiceClient.getAttribute(attribute);
    }

    /**
     * Check the interface consistency of {@link ConfigurationInstallerComponentClientImpl#setAttribute(String, Object)}
     */
    @Test
    public void testSetAttribute() throws AttributeErrorException {

        this.setAttribute(ConfigurationInstallerComponentService.ATTR_NAME, "value");
    }

    @Override
    public void setAttribute(final String name, final Object value) throws AttributeErrorException {

        this.configurationInstallerComponentServiceClient.setAttribute(name, value);
    }

    /**
     * Check the interface consistency of {@link ConfigurationInstallerComponentClientImpl#getAttributes()}
     */
    @Test
    public void testGetAttributes() throws AttributeErrorException {

        this.getAttributes();
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getAttributes() throws AttributeErrorException {

        return this.configurationInstallerComponentServiceClient.getAttributes();
    }

    /**
     * Check the interface consistency of {@link ConfigurationInstallerComponentClientImpl#setAttributes()}
     */
    @Test
    public void testSetAttributes() throws AttributeErrorException {

        this.setAttributes(new HashMap<MBeanAttributeInfo, Object>());
    }

    @Override
    public void setAttributes(final Map<MBeanAttributeInfo, Object> attributes) throws AttributeErrorException {

        this.configurationInstallerComponentServiceClient.setAttributes(attributes);
    }

}
