/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.jmx.api.api.EndpointRegistryClient;
import org.ow2.petals.jmx.api.impl.mbean.RegistryService;
import org.ow2.petals.jmx.api.impl.mbean.RegistryServiceMBean;

/**
 * <p>
 * Unit tests of {@link EndpointRegistryClientImpl} to check that JMX client interfaces are aligned with the JMX server
 * MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link RegistryServiceMBean} to avoid to forget some methods. A unit test will only invoke
 * the associated method of {@link RegistryServiceMBean}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class EndpointRegistryClientImplTest extends AbstractServiceClientImpl implements RegistryServiceMBean {

    private EndpointRegistryClient registryServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerRegistryService(new RegistryService());
        this.registryServiceClient = jmxClient.getEndpointRegistryClient();
    }

    /**
     * Check the interface consistency of {@link EndpointRegistryClientImpl#getAllInternalEndpoints()}
     */
    @Test
    public void testGetAllInternalEndpoints() throws RegistryException {
        this.getAllInternalEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllInternalEndpoints() throws RegistryException {
        return this.registryServiceClient.getAllInternalEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointRegistryClientImpl#getAllExternalEndpoints()}
     */
    @Test
    public void testGetAllExternalEndpoints() throws RegistryException {
        this.getAllExternalEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllExternalEndpoints() throws RegistryException {
        return this.registryServiceClient.getAllExternalEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointRegistryClientImpl#getAllEndpoints()}
     */
    @Test
    public void testGetAllEndpoints() throws RegistryException {
        this.getAllEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllEndpoints() throws RegistryException {
        return this.registryServiceClient.getAllEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointRegistryClientImpl#getDescription(String, String)}
     */
    @Test
    public void testGetDescription() throws RegistryException {

        this.getDescription("serviceName", "endpointName");

    }

    @Override
    public String getDescription(final String serviceName, final String endpointName) throws RegistryException {
        return this.registryServiceClient.getDescription(serviceName, endpointName);
    }

}
