/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.RouterMonitorServiceClient;
import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceErrorException;
import org.ow2.petals.jmx.api.impl.mbean.RouterMonitorService;

/**
 * <p>
 * Unit tests of {@link RouterMonitorServiceClientImpl} to check that JMX client interfaces are aligned with the JMX
 * server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link RouterMonitorServiceClient} to avoid to forget some methods. A unit test will only
 * invoke the associated method of {@link RouterMonitorServiceClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RouterMonitorServiceClientImplTest extends AbstractServiceClientImpl implements RouterMonitorServiceClient {

    private RouterMonitorServiceClient routerMonitorServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerRouterMonitorService(new RouterMonitorService());
        this.routerMonitorServiceClient = jmxClient.getRouterMonitoringServiceClient();
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#clearMonitorStorage()}
     */
    @Test
    public void testClearMonitorStorage() throws RouterMonitorServiceErrorException {

        this.clearMonitorStorage();

    }

    @Override
    public void clearMonitorStorage() throws RouterMonitorServiceErrorException {
        this.routerMonitorServiceClient.clearMonitorStorage();
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#getExchange(String)}
     */
    @Test
    public void testGetExchange() throws RouterMonitorServiceErrorException {

        this.getExchange("exchange-id");

    }

    @Override
    public Map<String, String> getExchange(final String exchangeId) throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getExchange(exchangeId);
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#getExchangeHistory(String)}
     */
    @Test
    public void testGetExchangeHistory() throws RouterMonitorServiceErrorException {

        this.getExchangeHistory("exchange-id");

    }

    @Override
    public Map<String, Long> getExchangeHistory(final String exchangeId) throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getExchangeHistory(exchangeId);
    }

    /**
     * Check the interface consistency of
     * {@link RouterMonitorServiceClientImpl#getExchangeIds(QName, QName, String, QName, long, long)}
     */
    @Test
    public void testGetExchangeIds_1() throws RouterMonitorServiceErrorException {

        this.getExchangeIds(new QName("interface"), new QName("service"), "endpoint", new QName("operation"), 0, 0);

    }

    @Override
    public List<String> getExchangeIds(final QName interfaceName, final QName serviceName, final String endpointName,
            final QName operationName, final long startDate, final long endDate)
            throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getExchangeIds(interfaceName, serviceName, endpointName, operationName,
                startDate, endDate);
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#getExchangeIds(String, String)}
     */
    @Test
    public void testGetExchangeIds_2() throws RouterMonitorServiceErrorException {

        this.getExchangeIds("propertyKey", "propertyValue");

    }

    @Override
    public List<String> getExchangeIds(final String propertyKey, final String propertyValue)
            throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getExchangeIds(propertyKey, propertyValue);
    }

    /**
     * Check the interface consistency of
     * {@link RouterMonitorServiceClientImpl#getExchanges(QName, QName, String, QName, long, long)}
     */
    @Test
    public void testGetExchanges() throws RouterMonitorServiceErrorException {

        this.getExchanges(new QName("interface"), new QName("service"), "endpoint", new QName("operation"), 0, 0);

    }

    @Override
    public int getExchanges(final QName interfaceName, final QName serviceName, final String endpointName,
            final QName operationName, final long startDate, final long endDate)
            throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getExchanges(interfaceName, serviceName, endpointName, operationName,
                startDate, endDate);
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#getMonitoredExchange()}
     */
    @Test
    public void testGetMonitoredExchange() throws RouterMonitorServiceErrorException {

        this.getMonitoredExchange();

    }

    @Override
    public List<Map<String, String>> getMonitoredExchange() throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getMonitoredExchange();
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#getMonitorStorageDuration()}
     */
    @Test
    public void testGetMonitorStorageDuration() throws RouterMonitorServiceErrorException {

        this.getMonitorStorageDuration();

    }

    @Override
    public Long getMonitorStorageDuration() throws RouterMonitorServiceErrorException {
        return this.routerMonitorServiceClient.getMonitorStorageDuration();
    }

    /**
     * Check the interface consistency of
     * {@link RouterMonitorServiceClientImpl#monitorExchanges(QName, QName, String, QName, boolean)}
     */
    @Test
    public void testMonitorExchanges() throws RouterMonitorServiceErrorException {

        this.monitorExchanges(new QName("interface"), new QName("service"), "endpoint", new QName("operation"),
                Boolean.TRUE.booleanValue());

    }

    @Override
    public void monitorExchanges(final QName interfaceName, final QName serviceName, final String endpointName,
            final QName operationName, final boolean storeExchanges) throws RouterMonitorServiceErrorException {
        this.routerMonitorServiceClient.monitorExchanges(interfaceName, serviceName, endpointName, operationName,
                storeExchanges);
    }

    /**
     * Check the interface consistency of {@link RouterMonitorServiceClientImpl#setMonitorStorageDuration(long)}
     */
    @Test
    public void testSetMonitorStorageDuration() throws RouterMonitorServiceErrorException {

        this.setMonitorStorageDuration(0);

    }

    @Override
    public void setMonitorStorageDuration(final long duration) throws RouterMonitorServiceErrorException {
        this.routerMonitorServiceClient.setMonitorStorageDuration(duration);
    }

    /**
     * Check the interface consistency of
     * {@link RouterMonitorServiceClientImpl#unmonitorExchanges(QName, QName, String, QName)}
     */
    @Test
    public void testUnmonitorExchanges() throws RouterMonitorServiceErrorException {

        this.unmonitorExchanges(new QName("interface"), new QName("service"), "endpoint", new QName("operation"));

    }

    @Override
    public void unmonitorExchanges(final QName interfaceName, final QName serviceName, final String endpointName,
            final QName operationName) throws RouterMonitorServiceErrorException {
        this.routerMonitorServiceClient.unmonitorExchanges(interfaceName, serviceName, endpointName, operationName);
    }

}
