/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanAttributeInfo;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationErrorException;
import org.ow2.petals.jmx.api.impl.mbean.RuntimeConfigurationComponentService;

/**
 * <p>
 * Unit tests of {@link RuntimeConfigurationComponentClientImpl} to check that JMX client interfaces are aligned with
 * the JMX server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link RuntimeConfigurationComponentClient} to avoid to forget some methods. A unit test
 * will only invoke the associated method of {@link RuntimeConfigurationComponentClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RuntimeConfigurationComponentClientImplTest extends AbstractServiceClientImpl implements
        RuntimeConfigurationComponentClient {

    private RuntimeConfigurationComponentClient runtimeConfigurationComponentClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerRuntimeConfigurationComponentService(new RuntimeConfigurationComponentService(),
                COMPONENT_ID);
        this.runtimeConfigurationComponentClient = jmxClient.getRuntimeConfigurationComponentClient(COMPONENT_ID);
    }

    /**
     * Check the interface consistency of
     * {@link RuntimeConfigurationComponentClientImpl#getConfigurationMBeanAttributes()}
     */
    @Test
    public void testGetConfigurationMBeanAttributes() throws RuntimeConfigurationErrorException {

        this.getConfigurationMBeanAttributes();
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes() throws RuntimeConfigurationErrorException {

        return this.runtimeConfigurationComponentClient.getConfigurationMBeanAttributes();
    }

    /**
     * Check the interface consistency of {@link RuntimeConfigurationComponentClientImpl#setAttributes(Map)}
     */
    @Test
    public void testSetAttributes() throws RuntimeConfigurationErrorException {

        this.setAttributes(new HashMap<MBeanAttributeInfo, Object>());
    }

    @Override
    public void setAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws RuntimeConfigurationErrorException {

        this.runtimeConfigurationComponentClient.setAttributes(attributes);
    }

}
